@@ -17,9 +17,10 @@ along with GNU Make; see the file COPYING.  If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
+#include "make.h"
+
 #include <assert.h>
 
-#include "make.h"
 #include "job.h"
 #include "debug.h"
 #include "filedef.h"
@@ -74,7 +75,6 @@ static int amiga_batch_file;
 #endif /* Amiga.  */
 
 #ifdef VMS
-# include <time.h>
 # ifndef __GNUC__
 #   include <processes.h>
 # endif
@@ -295,10 +295,10 @@ vms_redirect (desc, fname, ibuf)
   extern char *vmsify ();
 
   ibuf++;
-  while (isspace (*ibuf))
+  while (isspace ((unsigned char)*ibuf))
     ibuf++;
   fptr = ibuf;
-  while (*ibuf && !isspace (*ibuf))
+  while (*ibuf && !isspace ((unsigned char)*ibuf))
     ibuf++;
   *ibuf = 0;
   if (strcmp (fptr, "/dev/null") != 0)
@@ -890,7 +890,7 @@ start_job_command (child)
 	flags |= COMMANDS_RECURSE;
       else if (*p == '-')
 	child->noerror = 1;
-      else if (!isblank (*p))
+      else if (!isblank ((unsigned char)*p))
 	break;
       ++p;
     }
@@ -898,20 +898,6 @@ start_job_command (child)
   /* Update the file's command flags with any new ones we found.  */
   child->file->cmds->lines_flags[child->command_line - 1] |= flags;
 
-  /* If -q was given, just say that updating `failed'.  The exit status of
-     1 tells the user that -q is saying `something to do'; the exit status
-     for a random error is 2.  */
-  if (question_flag && !(flags & COMMANDS_RECURSE))
-    {
-      child->file->update_status = 1;
-      notice_finished_file (child->file);
-      return;
-    }
-
-  /* There may be some preceding whitespace left if there
-     was nothing but a backslash on the first line.  */
-  p = next_token (p);
-
   /* Figure out an argument list from this command line.  */
 
   {
@@ -930,13 +916,31 @@ start_job_command (child)
       }
   }
 
+  /* If -q was given, say that updating `failed' if there was any text on the
+     command line, or `succeeded' otherwise.  The exit status of 1 tells the
+     user that -q is saying `something to do'; the exit status for a random
+     error is 2.  */
+  if (argv != 0 && question_flag && !(flags & COMMANDS_RECURSE))
+    {
+#ifndef VMS
+      free (argv[0]);
+      free ((char *) argv);
+#endif
+      child->file->update_status = 1;
+      notice_finished_file (child->file);
+      return;
+    }
+
   if (touch_flag && !(flags & COMMANDS_RECURSE))
     {
       /* Go on to the next command.  It might be the recursive one.
 	 We construct ARGV only to find the end of the command line.  */
 #ifndef VMS
-      free (argv[0]);
-      free ((char *) argv);
+      if (argv)
+        {
+          free (argv[0]);
+          free ((char *) argv);
+        }
 #endif
       argv = 0;
     }
@@ -968,8 +972,20 @@ start_job_command (child)
   message (0, (just_print_flag || (!(flags & COMMANDS_SILENT) && !silent_flag))
 	   ? "%s" : (char *) 0, p);
 
+  /* Tell update_goal_chain that a command has been started on behalf of
+     this target.  It is important that this happens here and not in
+     reap_children (where we used to do it), because reap_children might be
+     reaping children from a different target.  We want this increment to
+     guaranteedly indicate that a command was started for the dependency
+     chain (i.e., update_file recursion chain) we are processing.  */
+
+  ++commands_started;
+
   /* Optimize an empty command.  People use this for timestamp rules,
-     so avoid forking a useless shell.  */
+     so avoid forking a useless shell.  Do this after we increment
+     commands_started so make still treats this special case as if it
+     performed some action (makes a difference as to what messages are
+     printed, etc.  */
 
 #if !defined(VMS) && !defined(_AMIGA)
   if (
@@ -989,15 +1005,6 @@ start_job_command (child)
     }
 #endif  /* !VMS && !_AMIGA */
 
-  /* Tell update_goal_chain that a command has been started on behalf of
-     this target.  It is important that this happens here and not in
-     reap_children (where we used to do it), because reap_children might be
-     reaping children from a different target.  We want this increment to
-     guaranteedly indicate that a command was started for the dependency
-     chain (i.e., update_file recursion chain) we are processing.  */
-
-  ++commands_started;
-
   /* If -n was given, recurse to get the next line in the sequence.  */
 
   if (just_print_flag && !(flags & COMMANDS_RECURSE))
@@ -1411,7 +1418,8 @@ new_job (file)
 
 			  /* Discard any preceding whitespace that has
 			     already been written to the output.  */
-			  while (out > ref && isblank (out[-1]))
+			  while (out > ref
+				 && isblank ((unsigned char)out[-1]))
 			    --out;
 
 			  /* Replace it all with a single space.  */
@@ -1806,7 +1814,7 @@ child_execute_job (argv, child)
 
   DB (DB_JOBS, ("child_execute_job (%s)\n", argv));
 
-  while (isspace (*argv))
+  while (isspace ((unsigned char)*argv))
     argv++;
 
   if (*argv == 0)
@@ -1831,9 +1839,9 @@ child_execute_job (argv, child)
 	    p++;
 	    if (*p == '\n')
 	      p++;
-	    if (isspace (*p))
+	    if (isspace ((unsigned char)*p))
 	      {
-		do { p++; } while (isspace (*p));
+		do { p++; } while (isspace ((unsigned char)*p));
 		p--;
 	      }
 	    *q = *p;
@@ -1993,11 +2001,11 @@ child_execute_job (argv, child)
             case '\n':
               /* At a newline, skip any whitespace around a leading $
                  from the command and issue exactly one $ into the DCL. */
-              while (isspace (*p))
+              while (isspace ((unsigned char)*p))
                 p++;
               if (*p == '$')
                 p++;
-              while (isspace (*p))
+              while (isspace ((unsigned char)*p))
                 p++;
               fwrite (p, 1, q - p, outfile);
               fputc ('$', outfile);
@@ -2397,9 +2405,9 @@ construct_command_argv_internal (line, restp, shell, ifs, batch_filename_ptr)
   static char *sh_cmds_dos[] = { "break", "call", "cd", "chcp", "chdir", "cls",
 			     "copy", "ctty", "date", "del", "dir", "echo",
 			     "erase", "exit", "for", "goto", "if", "if", "md",
-			     "mkdir", "path", "pause", "prompt", "rem", "ren",
-			     "rename", "set", "shift", "time", "type",
-			     "ver", "verify", "vol", ":", 0 };
+			     "mkdir", "path", "pause", "prompt", "rd", "rem",
+                             "ren", "rename", "rmdir", "set", "shift", "time",
+                             "type", "ver", "verify", "vol", ":", 0 };
   static char sh_chars_sh[] = "#;\"*?[]&|<>(){}$`^";
   static char *sh_cmds_sh[] = { "cd", "eval", "exec", "exit", "login",
 			     "logout", "set", "umask", "wait", "while", "for",
@@ -2444,7 +2452,7 @@ construct_command_argv_internal (line, restp, shell, ifs, batch_filename_ptr)
     *restp = NULL;
 
   /* Make sure not to bother processing an empty line.  */
-  while (isblank (*line))
+  while (isblank ((unsigned char)*line))
     ++line;
   if (*line == '\0')
     return 0;
@@ -2962,12 +2970,12 @@ construct_command_argv (line, restp, file, batch_filename_ptr)
   for (;;)
     {
       while ((*cptr != 0)
-	     && (isspace (*cptr)))
+	     && (isspace ((unsigned char)*cptr)))
 	cptr++;
       if (*cptr == 0)
 	break;
       while ((*cptr != 0)
-	     && (!isspace(*cptr)))
+	     && (!isspace((unsigned char)*cptr)))
 	cptr++;
       argc++;
     }
@@ -2981,14 +2989,14 @@ construct_command_argv (line, restp, file, batch_filename_ptr)
   for (;;)
     {
       while ((*cptr != 0)
-	     && (isspace (*cptr)))
+	     && (isspace ((unsigned char)*cptr)))
 	cptr++;
       if (*cptr == 0)
 	break;
       DB (DB_JOBS, ("argv[%d] = [%s]\n", argc, cptr));
       argv[argc++] = cptr;
       while ((*cptr != 0)
-	     && (!isspace(*cptr)))
+	     && (!isspace((unsigned char)*cptr)))
 	cptr++;
       if (*cptr != 0)
 	*cptr++ = 0;
