@@ -853,6 +853,13 @@ int main (int argc, char ** argv)
   FATAL_SIG (SIGINT);
   FATAL_SIG (SIGTERM);
 
+#ifdef __MSDOS__
+  /* Windows 9X delivers FP exceptions in child programs to their
+     parent!  We don't want Make to die when a child divides by zero,
+     so we work around that lossage by catching SIGFPE.  */
+  FATAL_SIG (SIGFPE);
+#endif
+
 #ifdef	SIGDANGER
   FATAL_SIG (SIGDANGER);
 #endif
@@ -1195,7 +1202,7 @@ int main (int argc, char ** argv)
   /* Figure out the level of recursion.  */
   {
     struct variable *v = lookup_variable ("MAKELEVEL", 9);
-    if (v != 0 && *v->value != '\0' && *v->value != '-')
+    if (v != 0 && v->value[0] != '\0' && v->value[0] != '-')
       makelevel = (unsigned int) atoi (v->value);
     else
       makelevel = 0;
@@ -1255,17 +1262,47 @@ int main (int argc, char ** argv)
 	       and thus re-read the makefiles, we read standard input
 	       into a temporary file and read from that.  */
 	    FILE *outfile;
+            char *template, *tmpdir;
 
             if (stdin_nm)
               fatal (NILF, _("Makefile from standard input specified twice."));
 
 #ifdef VMS
-# define TMP_TEMPLATE   "sys$scratch:GmXXXXXX"
+# define DEFAULT_TMPDIR     "sys$scratch:"
 #else
-# define TMP_TEMPLATE   "/tmp/GmXXXXXX"
+# ifdef P_tmpdir
+#  define DEFAULT_TMPDIR    P_tmpdir
+# else
+#  define DEFAULT_TMPDIR    "/tmp"
+# endif
 #endif
+#define DEFAULT_TMPFILE     "GmXXXXXX"
 
-	    outfile = open_tmpfile (&stdin_nm, TMP_TEMPLATE);
+	    if (((tmpdir = getenv ("TMPDIR")) == NULL || *tmpdir == '\0')
+#if defined __MSDOS__ || defined(WINDOWS32)
+                /* These are also used commonly on these platforms.  */
+                && ((tmpdir = getenv ("TEMP")) == NULL || *tmpdir == '\0')
+                && ((tmpdir = getenv ("TMP")) == NULL || *tmpdir == '\0')
+#endif
+               )
+	      tmpdir = DEFAULT_TMPDIR;
+
+            template = (char *) alloca (strlen (tmpdir)
+                                        + sizeof (DEFAULT_TMPFILE) + 1);
+	    strcpy (template, tmpdir);
+
+#if defined __MSDOS__ || defined(WINDOWS32)
+	    if (strchr ("/\\", template[strlen (template) - 1]) == NULL)
+	      strcat (template, "/");
+#else
+#ifndef VMS
+	    if (template[strlen (template) - 1] != '/')
+	      strcat (template, "/");
+#endif /* !VMS */
+#endif /* __MSDOS__ || WINDOWS32 */
+
+	    strcat (template, DEFAULT_TMPFILE);
+	    outfile = open_tmpfile (&stdin_nm, template);
 	    if (outfile == 0)
 	      pfatal_with_name (_("fopen (temporary file)"));
 	    while (!feof (stdin))
@@ -1538,7 +1575,7 @@ int main (int argc, char ** argv)
 
   build_vpath_lists ();
 
-  /* Mark files given with -o flags as very old (00:00:01.00 Jan 1, 1970)
+  /* Mark files given with -o flags as very old
      and as having been updated already, and files given with -W flags as
      brand new (time-stamp as far as possible into the future).  */
 
@@ -1546,7 +1583,7 @@ int main (int argc, char ** argv)
     for (p = old_files->list; *p != 0; ++p)
       {
 	f = enter_command_line_file (*p);
-	f->last_mtime = f->mtime_before_update = (FILE_TIMESTAMP) 1;
+	f->last_mtime = f->mtime_before_update = OLD_MTIME;
 	f->updated = 1;
 	f->update_status = 0;
 	f->command_state = cs_finished;
@@ -1679,7 +1716,7 @@ int main (int argc, char ** argv)
                         error (NILF, _("Failed to remake makefile `%s'."),
                                d->file->name);
                         mtime = file_mtime_no_search (d->file);
-                        any_remade |= (mtime != (FILE_TIMESTAMP) -1
+                        any_remade |= (mtime != NONEXISTENT_MTIME
                                        && mtime != makefile_mtimes[i]);
                       }
                   }
@@ -2219,9 +2256,16 @@ decode_switches (argc, argv, env)
 		  break;
 
 		case positive_int:
-		  if (optarg == 0 && argc > optind
-                      && ISDIGIT (argv[optind][0]))
-		    optarg = argv[optind++];
+                  /* See if we have an option argument; if we do require that
+                     it's all digits, not something like "10foo".  */
+		  if (optarg == 0 && argc > optind)
+                    {
+                      const char *cp;
+                      for (cp=argv[optind]; ISDIGIT (cp[0]); ++cp)
+                        ;
+                      if (cp[0] == '\0')
+                        optarg = argv[optind++];
+                    }
 
 		  if (!doit)
 		    break;
@@ -2229,11 +2273,16 @@ decode_switches (argc, argv, env)
 		  if (optarg != 0)
 		    {
 		      int i = atoi (optarg);
-		      if (i < 1)
+                      const char *cp;
+
+                      /* Yes, I realize we're repeating this in some cases.  */
+                      for (cp = optarg; ISDIGIT (cp[0]); ++cp)
+                        ;
+
+		      if (i < 1 || cp[0] != '\0')
 			{
-			  if (doit)
-			    error (NILF, _("the `-%c' option requires a positive integral argument"),
-				   cs->c);
+                          error (NILF, _("the `-%c' option requires a positive integral argument"),
+                                 cs->c);
 			  bad = 1;
 			}
 		      else
@@ -2325,14 +2374,14 @@ decode_env_switches (envar, len)
     {
       if (*value == '\\' && value[1] != '\0')
 	++value;		/* Skip the backslash.  */
-      else if (isblank (*value))
+      else if (isblank ((unsigned char)*value))
 	{
 	  /* End of the word.  */
 	  *p++ = '\0';
 	  argv[++argc] = p;
 	  do
 	    ++value;
-	  while (isblank (*value));
+	  while (isblank ((unsigned char)*value));
 	  continue;
 	}
       *p++ = *value++;
@@ -2365,7 +2414,7 @@ quote_for_env (out, in)
     {
       if (*in == '$')
 	*out++ = '$';
-      else if (isblank (*in) || *in == '\\')
+      else if (isblank ((unsigned char)*in) || *in == '\\')
         *out++ = '\\';
       *out++ = *in++;
     }
@@ -2658,7 +2707,7 @@ print_version ()
 
   printf (_(", by Richard Stallman and Roland McGrath.\n\
 %sBuilt for %s\n\
-%sCopyright (C) 1988, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99\n\
+%sCopyright (C) 1988, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 2000\n\
 %s\tFree Software Foundation, Inc.\n\
 %sThis is free software; see the source for copying conditions.\n\
 %sThere is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n\
