@@ -39,11 +39,33 @@ Boston, MA 02111-1307, USA.  */
 # define PARAMS(protos)  ()
 #endif /* C++ or ANSI C.  */
 
+/* Specify we want GNU source code.  This must be defined before any
+   system headers are included.  */
+
+#define _GNU_SOURCE 1
+
+/* Include libintl.h, if it was found: we don't even look for it unless we
+   want to use the system's gettext().  If not, use the included gettext.h.  */
+
+#ifdef HAVE_LIBINTL_H
+# include <libintl.h>
+# ifdef HAVE_LOCALE_H
+#  include <locale.h>
+# endif
+#else
+# include "gettext.h"
+#endif
+
+#ifndef gettext_noop
+/* For automatic extraction of messages sometimes no real translation is
+   needed.  Instead the string itself is the result.  */
+# define gettext_noop(Str) (Str)
+#endif
 
-#include "gettext.h"
 #define _(Text)     gettext (Text)
 #define N_(Text)    gettext_noop (Text)
 
+
 #if !HAVE_SETLOCALE
 # define setlocale(Category, Locale) /* empty */
 #endif
@@ -60,7 +82,6 @@ Boston, MA 02111-1307, USA.  */
 # define __NO_STRING_INLINES
 #endif
 
-#define _GNU_SOURCE 1
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <signal.h>
@@ -72,7 +93,18 @@ Boston, MA 02111-1307, USA.  */
    <sys/timeb.h>?  If any does not, configure should check for it.  */
 # include <sys/timeb.h>
 #endif
-#include <time.h>
+
+#if TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #include <errno.h>
 
 #ifndef errno
@@ -275,14 +307,10 @@ extern char *alloca ();
 # endif /* HAVE_ALLOCA_H.  */
 #endif /* GCC.  */
 
-#ifdef ST_MTIM_NSEC
-# if HAVE_INTTYPES_H
-#  include <inttypes.h>
-# endif
-# define FILE_TIMESTAMP uintmax_t
-#else
-# define FILE_TIMESTAMP time_t
+#if HAVE_INTTYPES_H
+# include <inttypes.h>
 #endif
+#define FILE_TIMESTAMP uintmax_t
 
 /* ISDIGIT offers the following features:
    - Its arg may be any int or unsigned int; it need not be an unsigned char.
